import hashlib
from tqdm import tqdm_notebook
from IPython.core.display import display, HTML
from time import sleep


def init(name):
    print('[swiftace] Project initlalized. Key: 756135b7-632e-4956-adba-a13c7dd8f3c3')


def md5(fname):
    hash_md5 = hashlib.md5()
    with open(fname, "rb") as f:
        for chunk in iter(lambda: f.read(4096), b""):
            hash_md5.update(chunk)
    return hash_md5.hexdigest()


def log_dataset(path):
    for i in tqdm_notebook(range(100)):
        pass
    print('[swiftace] Dataset logged. Checksum: ' + md5(path))


def log_hyperparams(dict):
    for i in tqdm_notebook(range(100)):
        pass
    print(dict)
    print('')
    print('[swiftace] Hyperparameters logged successfully.')


class KerasCallback(object):
    def __init__(self):
        self.validation_data = None
        self.model = None

    def set_params(self, params):
        self.params = params

    def set_model(self, model):
        self.model = model

    def on_epoch_begin(self, epoch, logs=None):
        pass

    def on_epoch_end(self, epoch, logs=None):
        pass

    def on_batch_begin(self, batch, logs=None):
        pass

    def on_batch_end(self, batch, logs=None):
        pass

    def on_train_begin(self, logs=None):
        pass

    def on_train_end(self, logs=None):
        print('')
        print('[swiftace] Training metrics logged successfully.')


def log_metrics(dict):
    for _ in tqdm_notebook(range(100)):
        pass
    print(dict)
    print('')
    print('[swiftace] Metrics logged successfully.')


def upload_file(path):
    for _ in tqdm_notebook(range(21), unit='MB'):
        pass
    print('[swiftace] File uploaded successfully. Checksum: ' + md5(path))


def commit(message):
    for i in tqdm_notebook(range(4)):
        if i == 0:
            print('[swiftace] Capturing environment...')
        elif i == 1:
            print('[swiftace] Saving Jupyter notebook...')
        elif i == 2:
            print('[swiftace] Uploading source code...')
        else:
            print('[swiftace] Finalizing commit...')
        sleep(1)
    print('')
    print('[swiftace] Commit 0a9dc74d successful.')
    return display(HTML("""<a href="https://swiftace.ai/aakashns/keras-mnist/0a9dc74d" target="_blank">https://swiftace.ai/aakashns/mnist-basic/0a9dc74d</a>"""))



import requests

API_URL = 'https://swiftai-217215.appspot.com'


class ApiError(Exception):
    """Error class for web API related Exceptions"""
    pass


def upload_conda_env(data, name):
    """Upload the anaconda environment to server"""
    files = {'env': ('environment.yml', data)}
    res = requests.post(f'{API_URL}/environment',
                        files=files, data={'name': name})
    if res.status_code == 200:
        res_json = res.json()
        env_hash = res_json['hash']
        return env_hash
    else:
        raise ApiError(f'Evironment upload failed! {res.content}')


def download_conda_env(env_id):
    """Download an anaconda environment to a string"""
    res = requests.get(f'{API_URL}/environment/{env_id}')
    if res.status_code == 200:
        return res.content
    else:
        raise ApiError(f'Environment download failed! {res.content}')



import argparse
from swiftace.utils.anaconda import save_conda_env, load_conda_env


def exec_save_env():
    print('Capturing & saving anaconda environment..')
    hash = save_conda_env()
    print(f'Environment saved! ID: {hash}')
    print('Load using the command:')
    print(f'  $ swiftace load-env {hash}\n')


def exec_load_env(env_id, name):
    if not env_id:
        print('Environment ID is required!')
        return
    if not name:
        print('Please provide a name for the target environment using --name')
        return
    print(f'Loading environment "{env_id}"')
    load_conda_env(env_id, name)


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('command')
    parser.add_argument('env_id', nargs='?')
    parser.add_argument('-n', '--name')
    return parser.parse_args()


def main():
    args = get_args()
    command = args.command
    if command == 'save-env':
        exec_save_env()
    elif command == 'load-env':
        exec_load_env(args.env_id, args.name)
    else:
        print('Unknown command')


if __name__ == '__main__':
    main()
